package stog

  1. Overview
  2. Docs

Source file tags.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
(*********************************************************************************)
(*                Stog                                                           *)
(*                                                                               *)
(*    Copyright (C) 2012-2024 INRIA All rights reserved.                         *)
(*    Author: Maxence Guesdon, INRIA Saclay                                      *)
(*                                                                               *)
(*    This program is free software; you can redistribute it and/or modify       *)
(*    it under the terms of the GNU General Public License as                    *)
(*    published by the Free Software Foundation, version 3 of the License.       *)
(*                                                                               *)
(*    This program is distributed in the hope that it will be useful,            *)
(*    but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               *)
(*    GNU General Public License for more details.                               *)
(*                                                                               *)
(*    You should have received a copy of the GNU General Public                  *)
(*    License along with this program; if not, write to the Free Software        *)
(*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA                   *)
(*    02111-1307  USA                                                            *)
(*                                                                               *)
(*    As a special exception, you have permission to link this program           *)
(*    with the OCaml compiler and distribute executables, as long as you         *)
(*    follow the requirements of the GNU GPL in regard to all of the             *)
(*    software in the executable aside from the OCaml compiler.                  *)
(*                                                                               *)
(*    Contact: Maxence.Guesdon@inria.fr                                          *)
(*                                                                               *)
(*********************************************************************************)

(** *)

let site_title = "site-title"
let site_desc = "site-description"
let site_url = "site-url"
let site_email = "site-email"
let stog_dir = "stog-dir"
let rss_length = "rss-length"
let languages = "languages"
let functions = "functions_"
let default_style = "default-style"

let doc = "doc"
let doc_body = "doc-body"
let doc_date = "doc-date"
let doc_datetime = "doc-datetime"
let doc_intro = "doc-intro"
let doc_keywords = "doc-keywords"
let doc_navpath = "doc-navpath"
let doc_path = "doc-path"
let doc_src = "doc-src"
let doc_title = "doc-title"
let doc_topics = "doc-topics"
let doc_type = "doc-type"
let doc_url = "doc-url"

let sep = "sep_"

let archive_tree = "archive-tree"
let as_cdata = "as-cdata"
let as_xml = "as-xml"
let block = "block"
let command_line = "command-line"
let counter = "counter"
let date_now = "date-now"
let date_today = "date-today"
let dummy_ = "dummy_"
let documents = "documents"
let exec = "exec"
let ext_a = "ext-a"
let error_ = "error_"
let graph = "graph"
let hcode = "hcode"
let path_sep = "path-sep"
let icode = "icode"
let if_ = "if"
let image = "image"
let inc = "inc"
let include_ = "include"
let keyword = "keyword"
let langswitch = "langswitch"
let late_cdata = "late-cdata"
let late_inc = "late-inc"
let latex = "latex"
let latex_body = "latex-body"
let list = "list"
let n_columns = "n-columns"
let next = "next"
let next_path = "next-path"
let ocaml = "ocaml"
let ocaml_eval = "ocaml-eval"
let ocaml_printf = "ocaml-printf"
let page = "page"
let paragraph = "paragraph"
let post = "post"
let prefix_svg_ids = "prefix-svg-ids"
let prepare_toc = "prepare-toc"
let previous = "previous"
let previous_path = "previous-path"
let print_date = "print-date"
let print_datetime = "print-datetime"
let search_form = "search-form"
let section = "section"
let subsection = "subsection"
let subsubsection = "subsubsection"
let toc = "toc"
let topic = "topic"
let two_columns = "two-columns"

let default_sectionning =
  [ section ;
    subsection ;
    subsubsection ;
    paragraph ;
  ]